/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.beans.IntrospectionException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import nl.cloudfarming.client.model.PartField;
import nl.cloudfarming.client.model.ProductionUnit;
import nl.cloudfarming.client.planning.PlanEventKey;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventBus;
import nl.cloudfarming.eventbus.GuiEventKey;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;

/**
 * Factory for PartFieldNode
 * @author Merijn Zengers
 */
public class PartFieldNodeFactory extends ChildFactory<PartField> implements PropertyChangeListener, GuiEventListener<Object> {

    private final ProductionUnitLayer layer;
    private ProductionUnit productionUnit;
    TacticalPlanningService tacticalPlanningService;

    public PartFieldNodeFactory(ProductionUnit productionUnit, ProductionUnitLayer layer) {
        this.productionUnit = productionUnit;
        this.layer = layer;
        init();
    }
    
    //only for test purpose
    protected void setService(TacticalPlanningService service){
        this.tacticalPlanningService = service;
    }

    public final void init() {
        productionUnit.addPropertyChangeListener(ProductionUnit.PROP_PARTFIELDS, this);
        tacticalPlanningService = Lookup.getDefault().lookup(TacticalPlanningServiceImpl.class);
        GuiEventBus.addListener(this);
    }

    @Override
    public void onEvent(GuiEvent event) {
        //Update the productionUnit. When a partField is linked this productionUnit is out of date.
        if(productionUnit != null && productionUnit.getId() != 0){
            this.productionUnit = tacticalPlanningService.findProductionUnitById(productionUnit.getId());
        }
        refresh(true);
    }

    @Override
    public boolean listensTo(GuiEventKey guiEventKey) {
        return guiEventKey.equals(PlanEventKey.LINK_FIELD_TO_PRODUCTION_UNIT);
    }

    @Override
    public String getModuleName() {
        return TacticalPlanningModule.MODULE_NAME;
    }

    /**
     * Creates Node based on bean. Name is field.node.name + id
     * @param key PartField which is wrapped in this node 
     * @return PartFieldNode which is created for this PartField
     */
    @Override
    protected Node createNodeForKey(PartField key) {
        try {
            ProductionUnitLayerObject layerObject = new ProductionUnitLayerObject(key);
            return new PartFieldNode(layer, layerObject,tacticalPlanningService);
        } catch (IntrospectionException ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    /**
     * Get all the keys based on all the available partFields in productionUnit
     * @param toPopulate List of partFields to which the new partfiels are added 
     * @return always true
     */
    @Override
    protected boolean createKeys(List<PartField> toPopulate) {
        toPopulate.addAll(productionUnit.getPartFields());
        return true;
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        refresh(true);
    }
}
