/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import javax.swing.JButton;
import nl.cloudfarming.client.model.crop.Crop;
import nl.cloudfarming.client.model.crop.CropService;
import nl.cloudfarming.client.model.crop.Race;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.message.api.Message;
import nl.cloudfarming.client.message.api.MessageEvent;
import nl.cloudfarming.client.message.api.MessageEventKey;
import nl.cloudfarming.client.message.api.MessageEventProducer;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.ProductionUnit;
import nl.cloudfarming.client.model.ProductionUnitAction;
import nl.cloudfarming.client.model.FieldPlan;
import nl.cloudfarming.client.planning.PlanAction;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/**
 * Plan a new ProductionUnit action.
 * Displays a dialog in which the user is able to plan a production unit
 * @author Timon Veenstra & Gerben Feenstra
 */
public class PlanProductionUnitController extends PlanAction implements ProductionUnitAction {

    // Static variables
    private static final AppLogger LOG = AppLogFactory.getLogger(TacticalPlanningModule.class);
    private static final String DISPLAY_NAME = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit_action.name.name");
    private static final String CONFIRM_MESSAGE_TITLE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit_action.confirm_message.title");
    private static final String CONFIRM_MESSAGE_FROM = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit_action.confirm_message.from");

    private static final String BUTTON_OK_TEXT = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit_action.name.ok");
    private static final String BUTTON_CANCEL_TEXT = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit_action.name.cancel");
    // Services
    private CropService cropservice = Lookup.getDefault().lookup(CropService.class);
    private TacticalPlanningService tacticalPlanningService = Lookup.getDefault().lookup(TacticalPlanningService.class);
    // Panel
    private PlanProductionUnitPanel panel;
    // Other variables
    private Dialog dlg;
//    private Crop selectedCrop;
    private JButton jButtonOK;
    private JButton jButtonCancel;

    /**
     * Constructor
     * Create an instance of this action
     */
    public PlanProductionUnitController() {
        putValue(NAME, DISPLAY_NAME);
    }

    /**
     * Get the PlanProductionUnit Panel
     * @return The Panel
     */
    public PlanProductionUnitPanel getPanel() {
        if (panel == null) {
            panel = new PlanProductionUnitPanel(this);
        }
        return panel;
    }

    /**
     * Get the Crop service from LookUp
     * @return The CropService
     */
    public CropService getService() {
        if (cropservice == null) {
            cropservice = Lookup.getDefault().lookup(CropService.class);
        }
        return cropservice;
    }

    /**
     * Set the cropService, mainly used for unittesting.
     * @param cropservice 
     */
    protected void setService(CropService cropservice) {
        this.cropservice = cropservice;
    }

    /**
     * Handle initial action to open up the panel and actions done by the comboboxes
     * @param e Actionevent e
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        
        // This class is registered as listener of CropComboBox and RaceComboBox
        if (e.getSource() == getPanel().getCropsComboBox()) { // CropComboBox
            // Puts new Races comboboxmodel into the races combobox on the panel
            Crop selectedCrop = (Crop) getPanel().getCropsComboBox().getSelectedItem();
            getPanel().populateRaces(getService().findRaces(selectedCrop));
        }  else {
            // Initial call to ActionPerformed (when this class is first called) will end up here.
            getPanel().populateCrops(getService().findCrops());
            Crop selectedCrop = (Crop) getPanel().getCropsComboBox().getSelectedItem();
            getPanel().populateRaces(getService().findRaces(selectedCrop));
            getOkButton().setEnabled(true);
            getPanel().getCropsComboBox().addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    Crop selectedCrop = (Crop) getPanel().getCropsComboBox().getSelectedItem();
                    getPanel().populateRaces(getService().findRaces(selectedCrop));
                }
            });
            //
            // create the dialog and add the ok and cancel buttons
            //
            DialogDescriptor dd = new DialogDescriptor(getPanel(), DISPLAY_NAME);
            dd.setOptions(new Object[]{getOkButton(), getCancelButton()});
            dlg = DialogDisplayer.getDefault().createDialog(dd);
            dlg.setVisible(true);
        }
    }

    /**
     * creates the dialog cancel button
     * @return Cancel button
     */
    private JButton getCancelButton() {
        if (jButtonCancel == null) {
            jButtonCancel = new JButton();
            jButtonCancel.setText(BUTTON_CANCEL_TEXT);
            jButtonCancel.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent ae) {
                    dlg.setVisible(false);
                }
            });
        }
        return jButtonCancel;
    }

    /**
     * Creates the dialog ok button
     *
     * @return
     */
    protected JButton getOkButton() {
        if (jButtonOK == null) {
            jButtonOK = new JButton();
            jButtonOK.setText(BUTTON_OK_TEXT);
            jButtonOK.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent ae) {
                    okButtonActionPerformed();
                }
            });
        }
        return jButtonOK;
    }

    /**
     * Method will be called when OK-button is pressed
     */
    private void okButtonActionPerformed() {
        dlg.setVisible(false);

        String puName = getPanel().getNameTextField().getText();
        Date puStartDate = getPanel().getStartDatePicker().getDate();
        Date puEndDate = getPanel().getEndDatePicker().getDate();


        // Get current selected fieldplan from lookup
        FieldPlan fieldPlan = Utilities.actionsGlobalContext().lookup(FieldPlan.class);

        assert (fieldPlan != null); // Should never happen, action can only be called on fieldplan.

        // Build PU
        Race selectedRace = (Race)getPanel().getRacesComboBox().getSelectedItem();
        ProductionUnit.Builder productionUnitBuilder = new ProductionUnit.Builder().setRace(selectedRace).setFieldPlan(fieldPlan).setName(puName).setStartDate(puStartDate).setEndDate(puEndDate);
        ProductionUnit productionUnit = tacticalPlanningService.createProductionUnit(productionUnitBuilder);

        StringBuilder fieldsSb = new StringBuilder();

        // Loop through selected fields. Create partfield for each, link PU to each.
        for (Object selectedField : getPanel().getAvailableFieldsList().getSelectedValues()) {
            if (selectedField instanceof Field) {
                Field field = (Field) selectedField;

                fieldsSb.append("- ");
                fieldsSb.append("(");
                fieldsSb.append(field.getId());
                fieldsSb.append(") ");
                fieldsSb.append(field.getName());
                fieldsSb.append("\n");

                tacticalPlanningService.createPartField(field, productionUnit);
            }
        }

        // Messaging
        // =========
        // Created production unit {0} ({1}) on fields [{2}] starting on {3} and ending on {4}    
        String confirm_message_text = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "field_plan.plan_production_unit_action.confirm_message.message_text", productionUnit.getName(), productionUnit.getRace().getCrop(), fieldsSb.toString(), productionUnit.getStartDate(), productionUnit.getEndDate());

        Message message = new Message();
        message.setTitle(CONFIRM_MESSAGE_TITLE);
        message.setMessageText(confirm_message_text);
        message.setOrigin(CONFIRM_MESSAGE_FROM);
        message.setDateReceived(new Date());
        new MessageEventProducer().triggerEvent(new MessageEvent(message, MessageEventKey.MESSAGE_NEW));
        LOG.info(confirm_message_text);
    }
}
