/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.beans.PropertyChangeListener;
import java.util.Date;
import java.util.List;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.model.*;
import nl.cloudfarming.client.model.crop.Crop;
import org.openide.nodes.Node;

/**
 *
 * @author Timon Veenstra
 */
public interface TacticalPlanningService extends ModelService, DataProvider {

    /**
     * Property for palettes
     * When a new palette gets introduced the change event will contain
     * null as old value and the palette as new value.
     * When a palettes gets removed, the change event will contain the palette
     * as old value and null as new value
     * 
     */
    public static final String PROP_PALETTES = "prop_palettes";
    /**
     * Creates a CropProtection within a transaction. Starts a transaction if it is not already started.
     * @param CropProtection used as parameter placeholder, use return value
     * @return
     */
    CropProtection createCropProtection(CropProtection.Builder cropProtectionBuilder);

    /**
     * Creates a CropProtectionTask within a transaction. Starts a transaction if it is not already started.
     * @param CropProtectionTask used as parameter placeholder, use return value
     * @return
     */
    CropProtectionTask createCropProtectionTask(CropProtectionTask.Builder cropProtectionTaskBuilder);

    /**
     * Creates a CropProtectionTask within a transaction. Starts a transaction if it is not already started.
     * @param CropProtectionTask used as parameter placeholder, use return value
     * @return
     */
    CropProtectionTask createCropProtectionTask(CropProtectionTask cropProtectionTask);

    
    /**
     * Creates an FieldPlan within a transaction. Starts a transaction if it is not already started.
     * @param year for which Fieldplan is created
     */
    FieldPlan createFieldPlanForYear(String name, int year);
   
    /**
     * Finds an Operation by its id
     * @param id: the id of the Operation which has to be found
     * @return if found Operation otherwise null
     */
    Operation findOperationById(long id);   
   
    /**
     * Finds the productionUnit by the supplied id
     * @param id: the id of the productionUnit which must be found
     * @return productionUnit
     */
    public ProductionUnit findProductionUnitById(long id);
   
    /**
     * Finds Task by the supplied id
     * @param id: the id of the Task which has to be found
     * @return if found Task else null
     */
    public Task findTaskById(long id);   

    void createNewPartField(final PartField field);

    /**
     * Creates an Operation within a transaction. Starts a transaction if it is not already started.
     * @param operation used as parameter placeholder, use return value
     * @return
     */
    Operation createOperation(Operation.Builder operationBuilder);

    /**
     * Creates an Operation within a transaction. Starts a transaction if it is not already started.
     * @param operation used as parameter
     * @return persisted Operation
     */
    Operation createOperation(Operation operation);

    /**
     * Create a PartField from a Field and a selected ProductionUnit
     *
     * The new PartField will inherit start- and end date from ProductionUnit and Acreage and WKT from the Field
     */
    PartField createPartField(Field field, ProductionUnit productionUnit);

    /**
     * Creates an Task within a transaction. Starts a transaction if it is not already started.
     * @param Task used as parameter placeholder, use return value
     * @return
     */
    Task createTask(Task.Builder taskBuilder);

    List<CropProtectionTask> findAllCropProtectionTasks();
   
    /**
     * Creates a productionUnit within a transaction. Starts a transaction if it is not already started.
     * @param productionUnitBuilder PUBuilder to build productionUnit from.
     * @return
     */
    ProductionUnit createProductionUnit(ProductionUnit.Builder productionUnitBuilder);
           
    /**
     * Find all the CropProtections
     * @return List<CropProtection> all the available CropProtections
     */
    List<CropProtection> findAllCropProtections();

    /**
     * Finds all the available Operations
     * @return List of Operation
     */
    List<Operation> findAllOperations();

    /**
     * Find all the available Fields
     * @return List Field
     */
    List<PartField> findAllPartFields();

    /**
     * Finds all available productionUnits
     * @return List of productionUnits
     */
    List<ProductionUnit> findAllProductionUnits();

    /**
     * Find all the available Tasks
     * @return List Task
     */
    List<Task> findAllTasks();

    /**
     * Todo: Merged in from base. Is this still being used?
     * find all fieldplans
     *
     * @return List of FieldPlans
     */
    List<FieldPlan> findFieldPlans();

    /**
     * Find Productionunits where Date date is between startDate and endDate
     * @param date The date which must be between startDate and endDate
     * @return List<ProductionUnit> the ProductionUnits which match.
     */
    List<ProductionUnit> findProductionUnitsByDate(Date date);

    List<ProductionUnit> findProductionUnitsByFieldPlan(FieldPlan fieldPlan);

    /**
     * Finds all the operations which are connected to the productionUnit via task
     * @param id the id of the productionUnit
     * @return List List of operations linked to the specified productionUnit
     */
    List<Task> findTaskByProductionUnitId(long id);

    Node[] getFieldplanNodes();

    /**
     * create a new part field name by looking for the amount of part fields based on a field and field plan
     *
     * @param field Field to create a new name from for part field
     * @param fieldPlan Field plan to create a new name from for the part field
     * @return A name for new a part field (fieldName + number of usage of part field)
     * this method has default access because it is called directly by a test
     */
    String getNewPartfieldName(Field field, FieldPlan fieldPlan);

    /**
     *
     * @return returns number of fieldPlans
     */
    int getNumberOfFieldPlans();

    /**
     * Removes productionUnit from fieldplan. Checks if the productionUnit which will be removed is managed by the enttity manager
     * @param productionUnit which will be removed.
     */
    void removeProductionUnitFromFieldPlan(ProductionUnit productionUnit);

    /**
     * Updates a Operation.
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    void updateOperation(Operation toBeMerged);

    /**
     * updates a productionUnit
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    void updateProductionUnit(ProductionUnit toBeMerged);

    /**
     * updates a task
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    void updateTask(Task toBeMerged);
    
    /**
     * Retrieve a color palette for a crop.
     * 
     * @param crop
     * @return 
     */
    Palette<Object> getCropPalette(Crop crop);
    
    /**
     * Add a property change listener
     * @param pcl 
     */
    void addPropertyChangeListener(PropertyChangeListener pcl);
    
    /**
     * remove a property change listener
     * @param pcl 
     */
    void removePropertyChangeListener(PropertyChangeListener pcl);

    Palette[] getPalettes();
   
}